///////////////////////////////////////////////////////////////////////////////
//	FILENAME:	eTAudio.h 
//	SUMMARY:	Interface for Audio annotations to eText documents
//	SUPERCLASS:	eTAudio:eTImage:Object
//	INTERFACE:	None
//	PROTOCOLS:	<DocNotification, Annotation, Tool, ASCIISupport, HTMDSupport,
//				LaTeXSupport, eTAudioNotification, InspectableTarget>
//	AUTHOR:		Rohit Khare
//	COPYRIGHT:	(c) 1994 California Institure of Technology, eText Project
///////////////////////////////////////////////////////////////////////////////
//	DESCRIPTION
//		an eTAudio holds an eTAudioComponent and manages its state.
///////////////////////////////////////////////////////////////////////////////
//	HISTORY
//	10/30/94:	Modified to support <InspectableTarget>
//	07/24/94:	Rewritten for PR1 release/eTImage
//  07/07/94:	Rewritten as a subclass of ImageAnnotation
//	06/18/94:	HTMDSupport. RK & TRZ.
//	01/24/94:	Created. Derived largely from Version3's temporary Hypertext
//				'93 hack. Renamed as eTAudio and eTAudioUI.
///////////////////////////////////////////////////////////////////////////////

#import "eTextKernel.h"
#import "eTImage.subproj/eTImageUI.h"
#import "eTImage.subproj/eTImage.h"
#import "eTAudioUI.h"

@protocol eTAudioNotification
- audioStarted;
- audioPaused;
- audioResumed;
- audioEnded;
@end

@interface eTAudio:eTImage <DocNotification, Annotation, Tool, ASCIISupport, HTMDSupport, LaTeXSupport, eTAudioNotification>
{
    id	etAudioComponent;
	id	audioDelegate;
}

- sound;
- (BOOL)isAudioMutable;
- (BOOL)isAudioLinked;
- (const char *)audioName;
- setAudioName:(const char *)newName;
- setAudioDelegate:(id <eTAudioNotification>) newDelegate;
- audioDelegate;
@end